@@ -66,8 +66,6 @@ pub enum Error {
     InvalidPyprojectTomlSyntax(#[from] toml_edit::TomlError),
     #[error("`pyproject.toml` does not match the required schema. When the `[project]` table is present, `project.name` must be present and non-empty.")]
     InvalidPyprojectTomlSchema(#[from] toml_edit::de::Error),
-    #[error("Editable installs with setup.py legacy builds are unsupported, please specify a build backend in pyproject.toml")]
-    EditableSetupPy,
     #[error("Failed to resolve requirements from {0}")]
     RequirementsResolve(&'static str, #[source] AnyErrorBuild),
     #[error("Failed to install requirements from {0}")]
@@ -100,7 +98,6 @@ impl IsBuildBackendError for Error {
             | Self::InvalidSourceDist(_)
             | Self::InvalidPyprojectTomlSyntax(_)
             | Self::InvalidPyprojectTomlSchema(_)
-            | Self::EditableSetupPy
             | Self::RequirementsResolve(_, _)
             | Self::RequirementsInstall(_, _)
             | Self::Virtualenv(_)
