@@ -98,7 +98,7 @@ impl schemars::JsonSchema for IndexUrl {
         schemars::schema::SchemaObject {
             instance_type: Some(schemars::schema::InstanceType::String.into()),
             metadata: Some(Box::new(schemars::schema::Metadata {
-                description: Some("The URL of an index to use for fetching packages (e.g., `https://pypi.org/simple`).".to_string()),
+                description: Some("The URL of an index to use for fetching packages (e.g., `https://pypi.org/simple`), or a local path.".to_string()),
               ..schemars::schema::Metadata::default()
             })),
             ..schemars::schema::SchemaObject::default()
@@ -413,7 +413,7 @@ impl<'a> IndexLocations {
 
 impl From<&IndexLocations> for UrlAuthPolicies {
     fn from(index_locations: &IndexLocations) -> UrlAuthPolicies {
-        UrlAuthPolicies::from_tuples(index_locations.indexes().map(|index| {
+        UrlAuthPolicies::from_tuples(index_locations.allowed_indexes().into_iter().map(|index| {
             let mut url = index
                 .url()
                 .root()
