@@ -87,7 +87,7 @@ impl EnvVars {
 
     /// Equivalent to the `--index-strategy` command-line argument.
     ///
-    /// For example, if set to `unsafe-any-match`, uv will consider versions of a given package
+    /// For example, if set to `unsafe-best-match`, uv will consider versions of a given package
     /// available across all index URLs, rather than limiting its search to the first index URL
     /// that contains the package.
     pub const UV_INDEX_STRATEGY: &'static str = "UV_INDEX_STRATEGY";
@@ -437,7 +437,7 @@ impl EnvVars {
     /// Used with `--python-platform macos` and related variants to set the
     /// deployment target (i.e., the minimum supported macOS version).
     ///
-    /// Defaults to `12.0`, the least-recent non-EOL macOS version at time of writing.
+    /// Defaults to `13.0`, the least-recent non-EOL macOS version at time of writing.
     pub const MACOSX_DEPLOYMENT_TARGET: &'static str = "MACOSX_DEPLOYMENT_TARGET";
 
     /// Disables colored output (takes precedence over `FORCE_COLOR`).
@@ -685,4 +685,7 @@ impl EnvVars {
 
     /// Equivalent to the `--torch-backend` command-line argument (e.g., `cpu`, `cu126`, or `auto`).
     pub const UV_TORCH_BACKEND: &'static str = "UV_TORCH_BACKEND";
+
+    /// Equivalent to the `--project` command-line argument.
+    pub const UV_PROJECT: &'static str = "UV_PROJECT";
 }
