@@ -63,6 +63,7 @@ pub(crate) async fn export(
     editable: EditableMode,
     locked: bool,
     frozen: bool,
+    include_annotations: bool,
     include_header: bool,
     script: Option<Pep723Script>,
     python: Option<String>,
@@ -259,6 +260,7 @@ pub(crate) async fn export(
                 &prune,
                 &extras,
                 &dev,
+                include_annotations,
                 editable,
                 hashes,
                 &install_options,
