@@ -48,8 +48,8 @@ fn help() {
                                  "UV_PYTHON_DOWNLOADS=never"]
 
     Global options:
-      -q, --quiet
-              Do not print any output
+      -q, --quiet...
+              Use quiet output
       -v, --verbose...
               Use verbose output
           --color <COLOR_CHOICE>
@@ -66,7 +66,7 @@ fn help() {
           --directory <DIRECTORY>
               Change to the given directory prior to running the command
           --project <PROJECT>
-              Run the command within the given project directory
+              Run the command within the given project directory [env: UV_PROJECT=]
           --config-file <CONFIG_FILE>
               The path to a `uv.toml` file to use for configuration [env: UV_CONFIG_FILE=]
           --no-config
@@ -127,8 +127,8 @@ fn help_flag() {
                                  "UV_PYTHON_DOWNLOADS=never"]
 
     Global options:
-      -q, --quiet
-              Do not print any output
+      -q, --quiet...
+              Use quiet output
       -v, --verbose...
               Use verbose output
           --color <COLOR_CHOICE>
@@ -145,7 +145,7 @@ fn help_flag() {
           --directory <DIRECTORY>
               Change to the given directory prior to running the command
           --project <PROJECT>
-              Run the command within the given project directory
+              Run the command within the given project directory [env: UV_PROJECT=]
           --config-file <CONFIG_FILE>
               The path to a `uv.toml` file to use for configuration [env: UV_CONFIG_FILE=]
           --no-config
@@ -205,8 +205,8 @@ fn help_short_flag() {
                                  "UV_PYTHON_DOWNLOADS=never"]
 
     Global options:
-      -q, --quiet
-              Do not print any output
+      -q, --quiet...
+              Use quiet output
       -v, --verbose...
               Use verbose output
           --color <COLOR_CHOICE>
@@ -223,7 +223,7 @@ fn help_short_flag() {
           --directory <DIRECTORY>
               Change to the given directory prior to running the command
           --project <PROJECT>
-              Run the command within the given project directory
+              Run the command within the given project directory [env: UV_PROJECT=]
           --config-file <CONFIG_FILE>
               The path to a `uv.toml` file to use for configuration [env: UV_CONFIG_FILE=]
           --no-config
@@ -335,8 +335,11 @@ fn help_subcommand() {
               Disable automatic downloads of Python. [env: "UV_PYTHON_DOWNLOADS=never"]
 
     Global options:
-      -q, --quiet
-              Do not print any output
+      -q, --quiet...
+              Use quiet output.
+              
+              Repeating this option, e.g., `-qq`, will enable a silent mode in which uv will write no
+              output to stdout.
 
       -v, --verbose...
               Use verbose output.
@@ -416,6 +419,8 @@ fn help_subcommand() {
               See `--directory` to change the working directory entirely.
               
               This setting has no effect when used in the `uv pip` interface.
+              
+              [env: UV_PROJECT=]
 
           --config-file <CONFIG_FILE>
               The path to a `uv.toml` file to use for configuration.
@@ -583,8 +588,11 @@ fn help_subsubcommand() {
               Disable automatic downloads of Python. [env: "UV_PYTHON_DOWNLOADS=never"]
 
     Global options:
-      -q, --quiet
-              Do not print any output
+      -q, --quiet...
+              Use quiet output.
+              
+              Repeating this option, e.g., `-qq`, will enable a silent mode in which uv will write no
+              output to stdout.
 
       -v, --verbose...
               Use verbose output.
@@ -664,6 +672,8 @@ fn help_subsubcommand() {
               See `--directory` to change the working directory entirely.
               
               This setting has no effect when used in the `uv pip` interface.
+              
+              [env: UV_PROJECT=]
 
           --config-file <CONFIG_FILE>
               The path to a `uv.toml` file to use for configuration.
@@ -724,8 +734,8 @@ fn help_flag_subcommand() {
                                  "UV_PYTHON_DOWNLOADS=never"]
 
     Global options:
-      -q, --quiet
-              Do not print any output
+      -q, --quiet...
+              Use quiet output
       -v, --verbose...
               Use verbose output
           --color <COLOR_CHOICE>
@@ -742,7 +752,7 @@ fn help_flag_subcommand() {
           --directory <DIRECTORY>
               Change to the given directory prior to running the command
           --project <PROJECT>
-              Run the command within the given project directory
+              Run the command within the given project directory [env: UV_PROJECT=]
           --config-file <CONFIG_FILE>
               The path to a `uv.toml` file to use for configuration [env: UV_CONFIG_FILE=]
           --no-config
@@ -796,8 +806,8 @@ fn help_flag_subsubcommand() {
                                  "UV_PYTHON_DOWNLOADS=never"]
 
     Global options:
-      -q, --quiet
-              Do not print any output
+      -q, --quiet...
+              Use quiet output
       -v, --verbose...
               Use verbose output
           --color <COLOR_CHOICE>
@@ -814,7 +824,7 @@ fn help_flag_subsubcommand() {
           --directory <DIRECTORY>
               Change to the given directory prior to running the command
           --project <PROJECT>
-              Run the command within the given project directory
+              Run the command within the given project directory [env: UV_PROJECT=]
           --config-file <CONFIG_FILE>
               The path to a `uv.toml` file to use for configuration [env: UV_CONFIG_FILE=]
           --no-config
@@ -952,8 +962,8 @@ fn help_with_global_option() {
                                  "UV_PYTHON_DOWNLOADS=never"]
 
     Global options:
-      -q, --quiet
-              Do not print any output
+      -q, --quiet...
+              Use quiet output
       -v, --verbose...
               Use verbose output
           --color <COLOR_CHOICE>
@@ -970,7 +980,7 @@ fn help_with_global_option() {
           --directory <DIRECTORY>
               Change to the given directory prior to running the command
           --project <PROJECT>
-              Run the command within the given project directory
+              Run the command within the given project directory [env: UV_PROJECT=]
           --config-file <CONFIG_FILE>
               The path to a `uv.toml` file to use for configuration [env: UV_CONFIG_FILE=]
           --no-config
@@ -1067,8 +1077,8 @@ fn help_with_no_pager() {
                                  "UV_PYTHON_DOWNLOADS=never"]
 
     Global options:
-      -q, --quiet
-              Do not print any output
+      -q, --quiet...
+              Use quiet output
       -v, --verbose...
               Use verbose output
           --color <COLOR_CHOICE>
@@ -1085,7 +1095,7 @@ fn help_with_no_pager() {
           --directory <DIRECTORY>
               Change to the given directory prior to running the command
           --project <PROJECT>
-              Run the command within the given project directory
+              Run the command within the given project directory [env: UV_PROJECT=]
           --config-file <CONFIG_FILE>
               The path to a `uv.toml` file to use for configuration [env: UV_CONFIG_FILE=]
           --no-config
